package data.hsqldb;

import java.sql.*;
import java.util.*;

import data.*;

public class AbteilungMapper extends AbstractMapper<Abteilung> {
	public final static String selectString = "select id,name from abteilung";
	public final static String insertString = "insert into abteilung (name) values (?)";
	public final static String updateString = "update abteilung set name=? where id=?";
	public final static String deleteString = "delete from abteilung where id=?";
	
	public AbteilungMapper(DataProviderHSQL data) {
		super(data);
	}

	@Override
	public Abteilung load(Integer id) {
		Abteilung a = (Abteilung) identity_map.get(id);
		if (a!=null) return a;
		
		Connection c = data.getConnection();
		
		try {
			PreparedStatement st = c.prepareStatement(selectString+" where id=?");
			st.setInt(1, id);
			List<Abteilung> l = (List<Abteilung>) load(st, new ArrayList<Abteilung>());
			if (l.isEmpty()) return null;
			a = l.get(0);
			identity_map.put(a.getId(), a);
			return a;
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}
	}

	@Override
	public void insert(Object o) {
		if (! (o instanceof Abteilung)) throw new RuntimeException();
		Abteilung a = (Abteilung) o;
		Connection connection = data.getConnection();
		if (a.getId()==null) {
			try {
				PreparedStatement st = connection.prepareStatement(insertString);
				st.setString(1, a.getName());
				st.executeUpdate();
				st = connection.prepareStatement("call IDENTITY()");
				ResultSet res = st.executeQuery();
				res.next();
				connection.commit();
				Integer id = res.getInt(1);
				identity_map.put(id, a);
				a.setId(id);
			} catch (SQLException exc) {
				exc.printStackTrace();
			}
		} else {
			update(a);
		}
	}

	@Override
	public void update(Object o) {
		if (! (o instanceof Abteilung)) throw new RuntimeException();
		Abteilung a = (Abteilung) o;
		Connection connection = data.getConnection();
		if (a.getId()!=null) {
			try {
			PreparedStatement st = connection.prepareStatement(updateString);
			st.setString(1, a.getName());
			st.setInt(2, a.getId());
			st.executeUpdate();
			connection.commit();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		} else {
			insert(a);
		}
	}

	@Override
	public void delete(Object o) {
		if (! (o instanceof Abteilung)) throw new RuntimeException();
		Abteilung a = (Abteilung) o;
		Connection connection = data.getConnection();
		if (a.getId()==null) throw new RuntimeException("p not persistent");
		try {
			PreparedStatement st = connection.prepareStatement(deleteString);
			st.setInt(1, a.getId());
			st.executeUpdate();
			identity_map.remove(a.getId());
			a.setId(null);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	@Override
	protected Abteilung load(ResultSet res) throws SQLException {
		Integer id = res.getInt("id");
		Abteilung a = (Abteilung) identity_map.get(id);
		if (a!=null) return a;
		a = new Abteilung();
		a.setId(id);
		a.setName(res.getString("name"));
		identity_map.put(a.getId(), a);
		return a;
	}
}
