package data.hsqldb;

import java.sql.*;
import java.util.*;

import data.*;

public class DataProviderHSQL implements DataProvider {
	private String jdbcConnectionString;
	private String user;
	private String password;
	private Connection connection;
	private Map<Class<?>, Mapper<?>> mapper = new HashMap<Class<?>, Mapper<?>>();
	
	DataProviderHSQL(String jdbcConnectionString, String user,
			String password) {
		this.jdbcConnectionString = jdbcConnectionString;
		this.user = user;
		this.password = password;
		
		mapper.put(Person.class, new PersonMapper(this));
		mapper.put(Abteilung.class, new AbteilungMapper(this));
	}

	@Override
	public void connect() {
		try {
			Class.forName("org.hsqldb.jdbc.JDBCDriver");
			connection = DriverManager.getConnection(jdbcConnectionString,  user,  password);
		} catch (Exception e) {
			connection = null;
		}		
	}

	@Override
	public void disconnect() {
		if (connection != null) {
			try {
				connection.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		connection = null;
	}

	@Override
	public <E> Mapper<E> getMapper(Class<E> c) {
		return (Mapper<E>) mapper.get(c);
	}

	@Override
	public Connection getConnection() {
		return connection;
	}

	@Override
	public void commit() {
		try {
			connection.commit();
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
}
