package data.hsqldb;

import java.sql.*;
import java.util.*;

import data.*;

public class PersonMapper extends AbstractMapper<Person> {
	public final static String selectString = "select id,vorname,nachname from person";
	public final static String insertString = "insert into person (vorname,nachname) values (?,?)";
	public final static String updateString = "update person set vorname=?, nachname=? where id=?";
	public final static String deleteString = "delete from person where id=?";
	
	public PersonMapper(DataProviderHSQL data) {
		super(data);
	}

	@Override
	public Person load(Integer id) {
		Person p = (Person) identity_map.get(id);
		if (p!=null) return p;
		
		Connection c = data.getConnection();
		
		try {
			PreparedStatement st = c.prepareStatement(selectString+" where id=?");
			st.setInt(1, id);
			List<Person> l = (List<Person>) load(st, new ArrayList<Person>());
			if (l.isEmpty()) return null;
			p = l.get(0);
			identity_map.put(p.getId(), p);
			return p;
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}
	}

	@Override
	public void insert(Object o) {
		if (! (o instanceof Person)) throw new RuntimeException();
		Person p = (Person) o;
		Connection connection = data.getConnection();
		if (p.getId()==null) {
			try {
				PreparedStatement st = connection.prepareStatement(insertString);
				st.setString(1, p.getVorname());
				st.setString(2, p.getNachname());
/*				if (p.getAbteilung()==null) {
					st.setNull(3, java.sql.Types.INTEGER);
				} else {
					st.setInt(3, p.getAbteilung().getId());					
				}*/
				st.executeUpdate();
				st = connection.prepareStatement("call IDENTITY()");
				ResultSet res = st.executeQuery();
				res.next();
				connection.commit();
				Integer id = res.getInt(1);
				identity_map.put(id, p);
				p.setId(id);
			} catch (SQLException exc) {
				exc.printStackTrace();
			}
		} else {
			update(p);
		}
	}

	@Override
	public void update(Object o) {
		if (! (o instanceof Person)) throw new RuntimeException();
		Person p = (Person) o;
		Connection connection = data.getConnection();
		if (p.getId()!=null) {
			try {
			PreparedStatement st = connection.prepareStatement(updateString);
			st.setString(1, p.getVorname());
			st.setString(2, p.getNachname());
/*			if (p.getAbteilung()==null) {
				st.setNull(3, java.sql.Types.INTEGER);
			} else {
				st.setInt(3,  p.getAbteilung().getId());
			}
*/			st.setInt(3, p.getId());
			st.executeUpdate();
			connection.commit();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		} else {
			insert(p);
		}
	}

	@Override
	public void delete(Object o) {
		if (! (o instanceof Person)) throw new RuntimeException();
		Person p = (Person) o;
		Connection connection = data.getConnection();
		if (p.getId()==null) throw new RuntimeException("p not persistent");
		try {
			PreparedStatement st = connection.prepareStatement(deleteString);
			st.setInt(1, p.getId());
			st.executeUpdate();
			identity_map.remove(p.getId());
			p.setId(null);
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	@Override
	protected Person load(ResultSet res) throws SQLException {
		Integer id = res.getInt("id");
		Person p = (Person) identity_map.get(id);
		if (p!=null) return p;
		p = new Person();
		p.setId(id);
		p.setVorname(res.getString("vorname"));
		p.setNachname(res.getString("nachname"));
		identity_map.put(p.getId(), p);
		return p;
	}
}
